#!/usr/bin/perl -w

# import libraries
require LWP::UserAgent;
#use LWP::Debug qw(+);
require HTTP::Request;
require HTTP::Response;
require HTML::Form;
require HTTP::Cookies;

# default values
my $url_server = "http://octopus.cbr.su.se/index.php";
my $username = "";
my $password = "";


# check if an input is allowed to be changed by users
sub IsAllowedInput {
	# param: input
	my( $input) = @_;
	# return: bool, if change of input by user is allowed
	my $allowed = 1;

	# do not allow hidden inputs
	if( $input->type eq "hidden") {
		$allowed = 0;
	}

	# do not allow buttons
	if( $input->type eq "button") {
		$allowed = 0;
	}

	# do not allow reset
	if( $input->type eq "reset") {
		$allowed = 0;
	}

	# do not allow submit
	if( $input->type eq "submit") {
		$allowed = 0;
	}

	# do not allow file
	if( $input->type eq "file") {
		$allowed = 0;
	}

	# do not allow image
	if( $input->type eq "image") {
		$allowed = 0;
	}

	# do not allow option
#	if( $input->type eq "option") {
#		$allowed = 0;
#	}

	# do not allow checkbox
#	if( $input->type eq "checkbox") {
#		$allowed = 0;
#	}

	# do not allow undefined names
	if( !defined( $input->name)) {
		$allowed = 0;
	}

	# do not allow readonly inputs
	if( $input->readonly) {
		$allowed = 0;
	}

	# do not allow disabled inputs
	if( $input->disabled) {
		$allowed = 0;
	}

	# input allowed
	return $allowed;
}


# create a new user agent
my $useragent = LWP::UserAgent->new;
# create http-style request message to $url_server, set $username, $password
my $request = HTTP::Request->new( GET => $url_server);
$request->authorization_basic( $username, $password);

# send request and get response (instance of HTTP::Response)
my $response = $useragent->request( $request);
# if no success, write error and exit
die( $response->status_line) if( !$response->is_success);

# DEBUG
#print( $response->content);

# parse $response and build form objects for each form element
# (a list of instances of HTMl::Form)
my @forms=HTML::Form->parse( $response);
# if more than one form, write error and exit
die( "Cannot handle anything but a single form") if( scalar( @forms) != 1);

# DEBUG
#print( $forms[ 0]->dump."\n");
#print( "ARGUMENTS: ");
#print( "@ARGV");
#print( "\n\n\n");

# loop through all form elements and change according to settings on command line
my @inputs = $forms[ 0]->inputs;
foreach( @inputs) {
	my $input = $_;

	# check if changing the input is allowed
	if( !IsAllowedInput( $input)) {
		next;
	}

	# check commandline: assemble what we look for
	my $grep_for = "--".$input->name;	#looks for "name" in the html source code and puts its value "sequence" in the variable "grep_for"
	# grep in the command line args
	my @grep_args = grep( /^$grep_for/, @ARGV); # search for "--sequence" in given command line
	# if parameter found
	if( scalar( @grep_args) > 0) {
		# extract new value, limit to maximal 2 elements, get second element
		my @grep_args_split = split( /=/, $grep_args[ 0], 2);	#splits only in 2 pieces
		my $new_value = $grep_args_split[ 1];

		# DEBUG
#		print( "NEW VALUE=".$new_value."\n");

		# check if $new_value is in possible values for current input
		my @list_possible_values = $input->possible_values;

		# loop through possible values to compare
		# remember if we found it
		my $value_matched = 0;
		foreach( @list_possible_values) {
			if( defined( $_)) {
				# DEBUG
				#print( "COMPARE ".$_."==".$new_value."\n");

				if( $_ eq $new_value) {
					# DEBUG
					#print( "YES\n");
					$input->value( $new_value);
					#$form[ 0]->value( $input->name, $new_value);
					$value_matched = 1;
					last;
				}
				else {
					# DEBUG
					#print( "NO\n");
				}
			}
		}
		# if value did not match any allowed, set to undefined
		if( $value_matched == 0 && !scalar( @list_possible_values) == 0) {
			#print( "DEBUG \$value_matched == 0\n");
			$input->value( undef);
			#$form[ 0]->value( $input->name, undef);
			#$form[ 0]->value( $input->name, "off");
		}
		# if no possible values listed, allow all
		if( scalar( @list_possible_values) == 0) {
			#print( "DEBUG scalar( \@list_possible_values) == 0\n");
			#print( "INPUT NAME=".$input->name."\n");
			#print( "NEW VALUE=".$new_value."\n");
			$input->value( $new_value);
			#$form[ 0]->value( $input->name, $new_value);
		}
	}

	# current value
	my $current_value = $input->value;
	if( !defined( $current_value)) {
		$current_value = "off";
	}

	# print name, set value, possible values
#	print( "--".$input->name."=".$current_value."  possible={");
	my @list_possible_values = $input->possible_values;
	foreach( @list_possible_values) {
		if( defined( $_)) {
			print( " ".$_);
		}
		else {
			print( " off");
		}
	}
#	print( "}\n");
} 

#DEBUG
#print( $forms[ 0]->dump."\n");

# send
$request = $forms[ 0]->click("dspo");
$request->authorization_basic( $username, $password);
$response = $useragent->request( $request);

# if no success, write error and exit
die( $response->status_line) if( !$response->is_success);

# DEBUG
print( $response->content);

# get result

