#!/usr/bin/perl

$protein = $ARGV[0];	#"database_all/1a0tP.jufo9d";

$infile = "/home/koehlej/projects/jufo/039_training_9D/database_all/".$protein.".jufo9d";
$out_file = "/home/koehlej/projects/jufo/039_training_9D/database_all/".$protein.".jufo_topo";

#read jufo output file
open (IN, $infile) || die "cannot open input file $file\n";
while ($in = <IN>){
	chomp $in;
	$in =~ s/^\s+//;
	@in = split /\s+/, $in;

	push @sequence, $in[1];

	#do 2-state baseline correction for TMhelix
	$TMH = $in[10] + ((7*0.111)/2);
	$noTMH = 0;
	foreach $i (4..9, 11..12){
		$noTMH += $in[$i];
	}
	$noTMH -= (7*0.111)/2;

	#do 2-state baseline correction for TMstrand
	$TMS = $in[11] + ((7*0.111)/2);
	$noTMS = 0;
	foreach $i (4..10, 12){
		$noTMS += $in[$i];
	}
	$noTMS -= (7*0.111)/2;

	#convert probabilities into topology
	$TMH >= $noTMH ? $TMHtopo = "H" : 
	$TMH <  $noTMH ? $TMHtopo = "-" : print "";

	$TMS >= $noTMS ? $TMStopo = "S" : 
	$TMS <  $noTMS ? $TMStopo = "-" : print "";

	#make the 2-state prediction exclusive: if it is helix, it can't be strand
	$TMHtopo eq "H" && $TMStopo eq "S" && $TMH > $TMS ? $TMStopo = "-" : 
	$TMHtopo eq "H" && $TMStopo eq "S" && $TMH < $TMS ? $TMHtopo = "-" : print "";

	#write topology array
	$TMHtopo eq "H" 		     ? push @topo, "H" : 
	$TMStopo eq "S" 		     ? push @topo, "S" :
	($TMHtopo eq "-" && $TMStopo eq "-") ? push @topo, "-" : print "";
}
close IN;

#remove kinks in copied array to be able to get length of SSE without kink
@topo_nokinks = @topo;
foreach $i (1..$#topo_nokinks+1){
	if ($topo[$i-1] eq "H" && $topo[$i] ne "H" && $topo[$i+1] eq "H"){
		$topo_nokinks[$i] = "H";
	}
	if ($topo[$i-1] eq "S" && $topo[$i] ne "S" && $topo[$i+1] eq "S"){
		$topo_nokinks[$i] = "S";
	}
}

#get length of TMspan as if no kinks were present (kink is 1 residue long)
$length = 0;
foreach $i (0..$#topo_nokinks){
	#only for beginning of the array
	if ($i == 0 && $topo_nokinks[$i] =~ /[A-Z]/){
		foreach $j ($i..$i+1000){
			$topo_nokinks[$j] =~ /[A-Z]/ ? $length++ : last;
		}
	}
	#get length starting at first residue of SSE
	elsif ($topo_nokinks[$i] =~ /[A-Z]/ && $topo_nokinks[$i-1] eq "-"){
		foreach $j ($i..$i+1000){
			$topo_nokinks[$j] =~ /[A-Z]/ ? $length++ : last;
		}
	}
	elsif ($topo_nokinks[$i] eq "-"){
		$length = 0;
	}

	push @length, $length;
}

#remove short TMspans
foreach $i (0..$#topo){
	#initialize TM improved topology array
	$new_topo[$i] = $topo[$i];

	#compare to array without kinks, remove strands shorter than 5 residues (optimized)
	if ($topo_nokinks[$i] eq "S" && $length[$i] < 5){
		$new_topo[$i] = "-";
		$topo_nokinks[$i] = "-";
	}
	#compare to array without kinks, remove helices shorter than 11 residues (optimized)
	elsif ($topo_nokinks[$i] eq "H" && $length[$i] < 11){
		$new_topo[$i] = "-";
		$topo_nokinks[$i] = "-";
	}
}

#print number of residues and sequence
open (OUT, ">".$out_file) || die "cannot open output file\n";
print OUT "number of residues:\t".($#sequence+1)."\n\n";
foreach $i (0..$#sequence){
	$tmp_seq .= $sequence[$i];
	$tmp_pred .= $new_topo[$i];

	if ($i%50==49 or $i==$#sequence){
		print OUT " $tmp_seq\n";
		print OUT " $tmp_pred\n\n";
		$tmp_seq = "";
		$tmp_pred = "";
	}

	elsif ($i%10==9){
		$tmp_seq .= " ";
		$tmp_pred .= " ";
	}

}
close OUT;

